import { Neo4jConnection, type DbConnection } from "ts-common";
import { queryService } from "../readers/queryService";
import * as fs from "fs";
import path from "path";
import { createInterface } from "readline";
import { Command } from "commander";

const program = new Command();

// Define CLI options 
program
  .name("manualMode")
  .description("Run manual Cypher queries against a Neo4j database")
  .option("-h, --host <string>", "Database host", "localhost")
  .option("-p, --port <number>", "Database port", "7687")
  .option("-u, --username <string>", "Database username", "neo4j")
  .option("-P, --password <string>", "Database password", "password")
  .option("-o, --output <string>", "Output file name", "output.json");

program.parse(process.argv);

const options = program.opts();

const rl = createInterface({
  input: process.stdin,
  output: process.stdout,
});

async function manualQuery(
  cypherQuery: string,
  outputFile: string,
  dbConnection: DbConnection,
) {
  const result = await queryService(dbConnection, cypherQuery);

  const outputPath = path.join(__dirname, outputFile);
  fs.writeFileSync(outputPath, JSON.stringify(result, null, 2));

  console.log(`Query results written to ${outputPath}`);
}

if (require.main === module) {

  const dbConfig: DbConnection = {
    id: 1,
    internalDatabaseName: "neo4j",
    url: `://${options.host}`,
    protocol: "bolt",
    port: parseInt(options.port),
    username: options.username,
    password: options.password,
    type: "neo4j",
  };

  console.log('Type a Cypher query line by line. Type "RUN" on a new line to execute the query.');
  
  let queryBuffer: string[] = []; // Buffer to store multi-line queries

  rl.on("line", async (line: string) => {
    if (line.trim().toLowerCase() === "exit") {
      console.log("Exiting...");
      rl.close();
      process.exit();
    } else if (line.trim().toLowerCase() === "run") {
      const fullQuery = queryBuffer.join("\n");
      queryBuffer = []; // Clear buffer for next input
      try {
        await manualQuery(fullQuery, outputFile, dbConfig);
      } catch (err) {
        console.error("Error executing query:", err);
      }
    } else {
      queryBuffer.push(line);
    }
  });
}

