import { getUserSaveState, type BackendQueryFormat, type GraphQueryResultFromBackend } from "ts-common";
import {
    Neo4jConnection, type DbConnection,
    RabbitMqBroker, RedisConnector
} from "ts-common";
import { RABBIT_HOST, RABBIT_PASSWORD, RABBIT_PORT, RABBIT_USER, USER_MANAGEMENT_SERVICE_API, type QueryExecutionTypes } from "../variables";
import { log } from "../logger";
import { QueryPublisher } from "../utils/queryPublisher";
import { query2Cypher } from "../utils/cypher/converter";
import { parseCypherQuery } from "../utils/cypher/queryParser";


export const queryService = async (db: DbConnection, query: string): Promise<GraphQueryResultFromBackend> => {
    // TODO: only neo4j is supported for now

    const connection = new Neo4jConnection(db);
    try {
        const [neo4jResult] = await connection.run([query]);
        const result = parseCypherQuery(neo4jResult.records);

        return result
    } catch (error) {
        log.error('Error parsing query result:', query, error);
        throw new Error('Error parsing query result');
    } finally {
        connection.close();
    }
}


export const queryServiceReader = async (frontendPublisher: RabbitMqBroker, mlPublisher: RabbitMqBroker, redis: RedisConnector, type: QueryExecutionTypes) => {
    if (type == null) {
        log.error('Unsupported query execution type:', type);
        throw new Error('Unsupported query execution type');
    }
    log.info('Starting query reader for', type);

    const publisher = new QueryPublisher(frontendPublisher, mlPublisher);

    const queryServiceConsumer = new RabbitMqBroker('requests-exchange', `${type}-query-queue`, `${type}-query-request`);
    await queryServiceConsumer.connect({
        protocol: 'amqp',
        hostname: RABBIT_HOST,
        port: RABBIT_PORT,
        username: RABBIT_USER,
        password: RABBIT_PASSWORD,
    });

    log.info('Connected to RabbitMQ ST!');

    await queryServiceConsumer.startConsuming<BackendQueryFormat>("query-service", async (message, headers) => {
        const ss = await getUserSaveState(headers.message.sessionData.userID, headers.message.sessionData.saveStateID, USER_MANAGEMENT_SERVICE_API);
        const routingKey = await redis.getRouting(headers.message.sessionData.sessionID)

        log.debug('Received query request:', message, headers, ss);
        log.debug('Received routing key:', routingKey);

        if (!headers.queryID) {
            log.error('QueryID not set in headers:', headers);
            return;
        }

        log.debug('Received query request:', message, headers);


        publisher.withHeaders(headers).withRoutingKey(routingKey.QueueID).withQueryID(headers.queryID);
        publisher.publishStatusToFrontend('Received');

        if (ss == null || ss.dbConnections == null || ss.dbConnections[0] == null || ss.dbConnections.length === 0) {
            log.error('Invalid SaveState received in queryServiceConsumer:', ss);
            publisher.publishErrorToFrontend('Invalid SaveState');
            return;
        }

        log.debug('translating query:', message.query);
        publisher.publishStatusToFrontend('Translating');

        const query = query2Cypher(message);
        if (query == null) {
            log.error('Error translating query:', message.query);
            publisher.publishErrorToFrontend('Error translating query');
            return;
        }

        log.debug('Translated query:', query);
        publisher.publishTranslationResultToFrontend(query);

        for (let i = 0; i < ss.dbConnections.length; i++) {
            queryService(ss.dbConnections[i], query).then((result) => {
                publisher.publishResultToFrontend(result);
                log.debug('Query result!');

                if (message.machineLearning && message.machineLearning.length > 0) {
                    for (let i = 0; i < message.machineLearning.length; i++) {
                        try {
                            publisher.publishMachineLearningRequest(result, message.machineLearning[i], headers);
                            log.debug('Published machine learning request', message.machineLearning[i]);
                        } catch (error) {
                            log.error('Error publishing machine learning request', error);
                            publisher.publishErrorToFrontend('Error publishing machine learning request');
                        }
                    }
                }

            }).catch((error) => {
                log.error('Error querying database', error);
                publisher.publishErrorToFrontend('Error querying database');
            });
        }
    });
}

