import { RabbitMqBroker, queryStatusUpdateMessageId, queryStatusErrorMessageId, type GraphQueryResultFromBackend, queryStatusResultMessageId, queryStatusTranslationResultMessageId, type GraphQueryTranslationResultMessage, type FromFrontendRequest, type ToMLMessage, type GraphQueryResultMessage, type MachineLearning } from "ts-common";
import { log } from "../logger";

export class QueryPublisher {
    private frontendPublisher: RabbitMqBroker;
    private mlPublisher: RabbitMqBroker;
    private routingKey?: string;
    private headers?: FromFrontendRequest;
    private queryID?: string;

    constructor(frontendPublisher: RabbitMqBroker, mlPublisher: RabbitMqBroker) {
        this.frontendPublisher = frontendPublisher;
        this.mlPublisher = mlPublisher;
    }

    withHeaders(headers?: FromFrontendRequest) {
        this.headers = headers;
        return this;
    }

    withRoutingKey(routingKey?: string) {
        this.routingKey = routingKey;
        return this;
    }

    withQueryID(queryID?: string) {
        this.queryID = queryID;
        return this;
    }

    publishStatusToFrontend(status: string) {
        if (!this.headers || !this.routingKey || !this.queryID) {
            throw new Error('Headers or RoutingKey or queryID not set');
        }

        this.frontendPublisher.publishMessageToFrontend({
            type: queryStatusUpdateMessageId,
            callID: this.headers.message.fromFrontend.callID,
            value: this.queryID,
            status: status
        }, this.routingKey, this.headers);
    }

    publishErrorToFrontend(reason: string) {
        if (!this.headers || !this.routingKey || !this.queryID) {
            throw new Error('Headers or RoutingKey or queryID not set');
        }

        this.frontendPublisher.publishMessageToFrontend({
            type: queryStatusErrorMessageId,
            callID: this.headers.message.fromFrontend.callID,
            value: this.queryID,
            status: reason
        }, this.routingKey, this.headers);
    }

    publishTranslationResultToFrontend(query: string) {
        if (!this.headers || !this.routingKey || !this.queryID) {
            throw new Error('Headers or RoutingKey or queryID not set');
        }

        this.frontendPublisher.publishMessageToFrontend({
            type: queryStatusTranslationResultMessageId,
            callID: this.headers.message.fromFrontend.callID,
            value: {
                result: query,
                queryID: this.headers.message.fromFrontend.callID
            },
            status: 'success'
        }, this.routingKey, this.headers);
    }

    publishResultToFrontend(result: GraphQueryResultFromBackend) {
        if (!this.headers || !this.routingKey || !this.queryID) {
            throw new Error('Headers or RoutingKey or queryID not set');
        }

        this.frontendPublisher.publishMessageToFrontend({
            type: queryStatusResultMessageId,
            callID: this.headers.message.fromFrontend.callID,
            value: {
                result: {
                    type: 'nodelink',
                    payload: result
                },
                queryID: this.headers.message.fromFrontend.callID
            },
            status: 'success'
        }, this.routingKey, this.headers);
    }

    publishMachineLearningRequest(result: GraphQueryResultFromBackend, mlAttributes: MachineLearning, headers: FromFrontendRequest) {
        if (!this.headers || !this.routingKey) {
            throw new Error('Headers or RoutingKey or queryID not set');
        }

        // FIXME: Change ML to use the same message format that the frontend uses
        const toMlResult = {
            nodes: result.nodes.map(node => ({ ...node, id: node._id })),
            edges: result.edges.map(edge => ({ ...edge, id: edge._id }))
        }

        const message: ToMLMessage = {
            queryID: headers.callid,
            type: "query_result",
            value: {
                type: 'nodelink',
                payload: toMlResult
            },
            mlAttributes: mlAttributes.parameters
        }

        let queueName = mlType2Queue[mlAttributes.type];
        if (!queueName) {
            log.error('Invalid ML type:', mlAttributes, mlAttributes.type);
            throw new Error('Invalid ML type');
        }

        this.mlPublisher.publishMessage({
            message: JSON.stringify(message),
            queueName,
            routingKey: queueName,
            options: {
                headers: {
                    ...headers,
                    message: Buffer.from(JSON.stringify(headers.message)),
                }
            }
        });

        log.debug('Published ML request:', queueName, headers.callid);
    }
}

const mlType2Queue = {
    centrality: "ctr_queue",
    linkPrediction: "lpr_queue",
    communityDetection: "cdt_queue",
    shortestPath: "stp_queue",
}
